!*************************************************************************************************
! Climate statistics recorder
!*************************************************************************************************
SUBROUTINE climate_stat &
   (tmp_air, tmp_soil, prec, par_grass,                   &
   year, doy, counter, phenology, pheno_clo, pheno_cls,   &
   C3g_no, C4g_no,                                        &
   tmp_air_RunningRecord,   prec_RunningRecord,           &
   tmp_soil_RunningRecord,  tmp_soil_month_ave,           &
   tmp_coldest_RunningRecord, tmp_coldest_20yr_ave,       &
   tmp_hottest_RunningRecord, tmp_hottest_20yr_ave,       &
   tmp_ave_C3growth_RunningRecord, tmp_recorder_C3growth, &
   tmp_ave_C4growth_RunningRecord, tmp_recorder_C4growth, &
   par_grass_RunningRecord                                )

USE data_structure
implicit none

!Define variables
real   ,intent(IN)::tmp_air   !2m air temperature (Celcius)
real   ,intent(IN)::tmp_soil  !soil temperature (Celcius)
real   ,intent(IN)::prec      !precipitation (mm day-1)
real   ,intent(IN)::par_grass !total PAR on top of the grass layer (micro mol photon m-2 s-1)

integer,intent(IN)::year      !
integer,intent(IN)::doy       !day of the year (1-365)
integer,intent(IN)::counter   !simulation day (1-Simulation_year*365)

integer,intent(IN)::C3g_no    !PFT number of C3 grass
integer,intent(IN)::C4g_no    !PFT number of C4 grass

logical,dimension(1:PFT_no),intent(IN)::phenology !Phenological status (T:foliation, F:dormance)
integer,dimension(1:PFT_no),intent(IN)::pheno_clo !counter number of last leaf onset
integer,dimension(1:PFT_no),intent(IN)::pheno_cls !counter number of last leaf shedding

real,dimension(365)::tmp_air_RunningRecord    !365days running record  of tmp_air (Celcius)
real,dimension(365)::prec_RunningRecord       !365days running record  of prec (mm/year)

real,dimension(365)::tmp_soil_RunningRecord   !30days running record  of tmp_soil (Celcius)
real                 tmp_soil_month_ave       !30days running average of tmp_soil (Celcius)

real,dimension(20)::tmp_coldest_RunningRecord !20yr running record  of tmp_coldest_month (Celcius)
real                tmp_coldest_20yr_ave      !20yr running average of tmp_coldest_month (Celcius)

real,dimension(20)::tmp_hottest_RunningRecord !20yr running record  of tmp_hottest_month (Celcius)
real                tmp_hottest_20yr_ave      !20yr running average of tmp_hottest_month (Celcius)

real,dimension(365)::par_grass_RunningRecord  !365 days running record of par_grass

real,dimension( 20)::tmp_ave_C3growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C3 grass (Celcius)
real,dimension( 20)::tmp_ave_C4growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C4 grass (Celcius)
real,dimension(  2)::tmp_recorder_C3growth
                 !temperature recorder for C3 growht phase of C3 grass
real,dimension(  2)::tmp_recorder_C4growth
                 !temperature recorder for C4 growht phase of C4 grass

!Define local variables
real tmp_air_month_ave      !monthly average of air temperature (Celcius)

!temperature recorder for C3 growht phase of C3 grass
   !on growth phase
   if ( phenology(C3g_no) ) then
      tmp_recorder_C3growth(1) = tmp_recorder_C3growth(1) + 1
      tmp_recorder_C3growth(2) = tmp_recorder_C3growth(2) + tmp_air
   endif
   
   if ( phenology(C4g_no) ) then
      tmp_recorder_C4growth(1) = tmp_recorder_C4growth(1) + 1
      tmp_recorder_C4growth(2) = tmp_recorder_C4growth(2) + tmp_air
   endif
   
   !on the day of last phenology day
   if ( counter == pheno_cls(C3g_no) ) then
      tmp_ave_C3growth_RunningRecord(2:20) = tmp_ave_C3growth_RunningRecord(1:19)
      tmp_ave_C3growth_RunningRecord(1)    = tmp_recorder_C3growth(2) / &
                                             max(tmp_recorder_C3growth(1), 1.0)
      tmp_recorder_C3growth(:) = 0.0
   endif
   
   if ( counter == pheno_cls(C4g_no) ) then
      tmp_ave_C4growth_RunningRecord(2:20) = tmp_ave_C4growth_RunningRecord(1:19)
      tmp_ave_C4growth_RunningRecord(1)    = tmp_recorder_C4growth(2) / &
                                             max(tmp_recorder_C4growth(1), 1.0)
      tmp_recorder_C4growth(:) = 0.0
   endif
   
!air temperature
   tmp_air_RunningRecord(2:365) = tmp_air_RunningRecord(1:364)
   tmp_air_RunningRecord(1)     = tmp_air
   
!soil temperature
   tmp_soil_RunningRecord(2:365) = tmp_soil_RunningRecord(1:364)
   tmp_soil_RunningRecord(1)     = tmp_soil
   tmp_soil_month_ave           = sum(tmp_soil_RunningRecord(1:30)) /30.0
   
!precipitation
   prec_RunningRecord(2:365) = prec_RunningRecord(1:364)
   prec_RunningRecord(1)     = prec
   
!coldest and hottest month of the year
   !update coldest and hottest month of the year
   if (Day_of_Month(doy) == Day_in_month(Month(doy))) then	!on the end of month
       tmp_air_month_ave = &
       sum( tmp_air_RunningRecord(1:Day_in_month(Month(doy))) ) / real( Day_in_month(Month(doy)) )
       
       if (Month(doy) == 1) then	!initialize on 31 January
           tmp_coldest_RunningRecord(2:20) = tmp_coldest_RunningRecord(1:19)
           tmp_coldest_RunningRecord(1)    = tmp_air_month_ave
           tmp_hottest_RunningRecord(2:20) = tmp_hottest_RunningRecord(1:19)
           tmp_hottest_RunningRecord(1)    = tmp_air_month_ave
       else
           tmp_coldest_RunningRecord(1) = min(tmp_coldest_RunningRecord(1), tmp_air_month_ave)
           tmp_hottest_RunningRecord(1) = max(tmp_hottest_RunningRecord(1), tmp_air_month_ave)
       end if
   end if
   
   !update 20 year running mean
   if (doy == 365) then	!on the end of year
      tmp_coldest_20yr_ave = sum( tmp_coldest_RunningRecord (1:min(20,year)) ) /min(20, year)
      tmp_hottest_20yr_ave = sum( tmp_hottest_RunningRecord (1:min(20,year)) ) /min(20, year)
   end if
   
   !running record of par_grass
   par_grass_RunningRecord(2:365) = par_grass_RunningRecord(1:364)
   par_grass_RunningRecord(1)     = par_grass
   
END SUBROUTINE climate_stat



!**************************************************************************************************
! Search existing Plant Functiona Type in the grid
!**************************************************************************************************
SUBROUTINE pft_present (pft_exist, tree_exist, pft, C3g_no, C4g_no)

USE data_structure
implicit none

!Define arguments
logical,dimension(1:PFT_no),intent(OUT)::pft_exist  !PFT exist or not (.TRUE. or .FALSE.)
logical,dimension(1:Max_no),intent(IN) ::tree_exist !tree exist or not (.TRUE. or .FALSE.) 
integer,dimension(1:Max_no),intent(IN) ::pft        !Plant Functional Type (1-PFT_no)
integer                    ,intent(IN) ::C3g_no     !PFT number of C3 grass
integer                    ,intent(IN) ::C4g_no     !PFT number of C4 grass

!Define local variables
integer	p, no	!loop counter

!Reset varialbes
pft_exist(:)      = .false.
pft_exist(C3g_no) = .true. 
pft_exist(C4g_no) = .true. 

!Servey for existance of woody PFTs
do no=1, Max_no
if (tree_exist(no)) then
    p            = pft(no)
	pft_exist(p) = .true. 
end if
end do

END SUBROUTINE pft_present



!*************************************************************************************************
! Biome detemination routine (Must be called @ doy=365)
! -- original criteria was taken from BIOME3 of A. Haxeltine & I.C. Prentice (1996) --
!*************************************************************************************************
SUBROUTINE biome_determine &
   (biome, C3g_no, C4g_no, W_fi, W_wilt, gdd0, gdd5, lai_max_wood, lai_max_grass, &
   npp_annual, pool_w1_RunningRecord)

USE data_structure
implicit none

!auguments
integer,intent(OUT)::biome         !biome type number
integer,intent(IN) ::C3g_no        !PFT number of C3 grass
integer,intent(IN) ::C4g_no        !PFT number of C4 grass
real   ,intent(IN) ::W_fi          !filed capacity   (m3/m3, 0.0 -> 1.0)
real   ,intent(IN) ::W_wilt        !wilting point    (m3/m3, 0.0 -> 1.0)
real   ,intent(IN) ::gdd0          !growing-degree-day-sum of 0 Celcius base
real   ,intent(IN) ::gdd5          !growing-degree-day-sum of 5 Celcius base
real   ,intent(IN) ::lai_max_wood  !Wood  LAI @ annual maximum  (m2/m2)
real   ,intent(IN) ::lai_max_grass !Grass LAI @ annual maximum  (m2/m2)
real,dimension(1:PFT_no),intent(IN)::npp_annual !NPP for each pft (g dm / stand / year)
real,dimension(365)     ,intent(IN)::pool_w1_RunningRecord  !1yr running record of pool_w1

!local variables
real,dimension(12)::sm     !soil moisture index for each month
real              ::sm_min !minimum value of sm
real              ::sm_ave !average value of sm

integer dominant1, dominant2 !
real    lai_max              !annual maximum of LAI (wood + grass)
integer p,i,j,k              !loop counter for temporal usage
real    x, y                 !for general usage

!----------- preparation -----------
   !calculate LAI max
   lai_max = lai_max_wood + lai_max_grass
   
   !determine first dominant PFT
   dominant1 = 1   !default
   x         = 0.0 
   do p = 1, PFT_no
      if (npp_annual(p) > x ) then
         x         = npp_annual(p)
         dominant1 = p
      endif
   enddo
   
   !determine secound dominant PFT
   dominant2 = 1   !default
   x         = 0.0 
   do p = 1, PFT_no
      if ( p .ne. dominant1 ) then
      if (npp_annual(p) > x ) then
         x         = npp_annual(p)
         dominant2 = p
      endif
      endif
   enddo
   
   !sm(:): soil moisture index of upper-soil-layer for each month
   do i = 1, 12                   !i: month
      k = sum(Day_in_month(1:i))  !k: ending doy of month i
      j = k - Day_in_month(i) + 1 !j: starting doy of month i
      x = sum(pool_w1_RunningRecord(j:k)) / real(k-j+1)
      sm(i) = (x/Depth1-W_wilt) / (W_fi-W_wilt)
      sm(i) = min( 1.0, max(0.0, sm(i)) )
   enddo
   sm_min = min( sm(1),sm(2),sm(3),sm(4),sm(5),sm(6),sm(7),sm(8),sm(9),sm(10),sm(11),sm(12) )
   sm_ave = sum( sm(1:12) ) / 12.0
   
!----------- Group 1 -----------
   !1: Polar desert
   if (gdd0 < 150) then
      biome = 1; return
   endif
   
!----------- Group 2 -----------
   !2: Arctic/Alpine-tundra
   if (gdd5 < 350) then
      biome = 2; return
   endif
   
!----------- Group 3 -----------
SELECT CASE (dominant1)
   !3: tropical rain forest
   !4: tropical seasonal forest
   CASE (1:4)
      if (lai_max>=2.5) then
         if (sm_min >= 0.50) then
            biome = 3; return
         else
            biome = 4; return
         endif
      endif
   
   !5: tropical deciduous forest
   CASE (5)
      if (lai_max>=2.5) then
         biome = 5; return
      endif
   
   !6: temperate conifer forest
   CASE (6)
      if (lai_max>=1.5) then
         biome = 6; return
      endif
   
   !7: temperate broad-leaved evergreen forest
   CASE (7)
      if (lai_max>=3.0) then
         biome = 7; return
      endif
   
   !8: temperate deciduous forest
   CASE (8)
      if (lai_max>=2.5 ) then
         biome = 8; return
      endif
   
   !9: boreal evergreen forest
   CASE (9)
         biome = 9; return
   
   !10: boreal deciduous forest
   CASE (10:11)
         biome = 10; return
   
   !11: short grass land
   !12: tall grass land
   CASE (12)
      if (lai_max_grass>1.5 .and. sm_ave<0.65) then
         if ( lai_max_grass<3.0 ) then
            biome = 11; return
         else
            biome = 12; return
         endif
      endif
   
   CASE (13)
      x = npp_annual(C3g_no) + npp_annual(C4g_no) !annual grass NPP
      y = max( 1.0 , sum(npp_annual(:))-x )     !annual woody NPP
      x = x / y                                 !annual grass-NPP / woody-NPP
      
      if (lai_max_grass>0.4 .and. x>1.8) then
         if ( lai_max_grass<3.0 ) then
            biome = 11; return
         else
            biome = 12; return
         endif
      endif
   
END SELECT

!----------- Group 4 -----------
   !13: moist savannas
   if ( (dominant1==C3g_no .or. dominant1==C4g_no) .and. lai_max>=1.5 )then
      biome = 13; return
   endif
   
   !14: dry savannas
   if ( (dominant1==C3g_no .or. dominant1==C4g_no) .and. lai_max>=0.6 )then
      biome = 14; return
   endif
   
   !15: xeric woodland / scrub
   if ( dominant1==1 .or. dominant1==2 .or. dominant1==3 .or. dominant1==4 .or. dominant1==5 .or. dominant1==7) then
   if ( lai_max>=1.0 ) then
      biome = 15; return
   endif
   endif
   
   if ( dominant1==6 .or. dominant1==7 .or. dominant1==9 .or. dominant1==10 .or. dominant1==11) then
   if ( lai_max>=1.5 ) then
      biome = 15; return
   endif
   endif
   
!----------- Group 5 -----------
   !16: Arid shrubland / steppe
   !17: Desert
   if ( lai_max>=0.2 ) then
      biome = 16
   else
      biome = 17
   endif
   
END SUBROUTINE biome_determine



!*************************************************************************************************
! Exchange variables
!*************************************************************************************************
!Real variables
   SUBROUTINE exchange_r(a,b)
   real    a, b !input variables to exchange
   real    x    !for temporal usage
   x = a
   a = b
   b = x
   END SUBROUTINE
   
!Integer variables
   SUBROUTINE exchange_i(a,b)
   integer    a, b !input variables to exchange
   integer    i    !for temporal usage
   i = a
   a = b
   b = i
   END SUBROUTINE
   
!Logical variables
   SUBROUTINE exchange_l(a,b)
   logical    a, b !input variables to exchange
   logical    l    !for temporal usage
   l = a
   a = b
   b = l
   END SUBROUTINE



!*************************************************************************************************
! output spinup files
!*************************************************************************************************
SUBROUTINE spinup_out &
   (type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd, &
   phenology, pft_exist, &
   tree_exist, &
   pool_w1, pool_w2, pool_w3, pool_snow, &
   pool_litter, pool_som_int, pool_som_slow, &
   canopy_cond, gdd0, gdd5, &
   establish_regu, mort_regu3, moist_factor, fire_factor, &
   c_emission, c_uptake, &
   pheno_gdd, tmp_soil_month_ave, &
   tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr, &
   lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
   pheno_clo, pheno_cls, &
   stat_water, npp_10d_ave, &
   pft, age, height, bole, height_limit, &
   dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, &
   la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, crown_x, crown_y, radius_limit, &
   stat_leaf, stat_leaf_crowntop, mort_regu1, mort_regu2, &
   stat_leaf_crownbottom, stat_leaf_RunningRecord, &
   npp_RunningRecord, stat_water_RunningRecord, &
   aet, gdd_20yr_RunningRecord, tmp_coldest_RunningRecord, tmp_hottest_RunningRecord, &
   tmp_ave_C3growth_RunningRecord, tmp_ave_C4growth_RunningRecord, &
   par_RunningRecord, tmp_soil_RunningRecord, tmp_air_RunningRecord, &
   prec_RunningRecord, pool_w1_RunningRecord, par_grass_RunningRecord, &
   tmp_recorder_C3growth, tmp_recorder_C4growth, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_grass, lai_opt_grass, &
   lai_opt_grass_RunningRecord)
   

USE data_structure
implicit none

!************************   DEFINE AUGMENTS   *************************
integer type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd

logical,dimension(PFT_no)::phenology, pft_exist
logical,dimension(1:Max_no)::tree_exist

real pool_w1, pool_w2, pool_w3, pool_snow
real pool_litter, pool_som_int, pool_som_slow
real canopy_cond, gdd0, gdd5
real establish_regu, mort_regu3, moist_factor, fire_factor
real c_emission, c_uptake
real pheno_gdd, tmp_soil_month_ave
real tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave
real hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr
real lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass

integer,dimension(PFT_no)::pheno_clo, pheno_cls
real   ,dimension(PFT_no)::stat_water, gpp, npp, gpp_month, npp_month
real   ,dimension(PFT_no)::gpp_annual, npp_annual, npp_10d_ave

integer,dimension(1:Max_no)::pft, age, height, bole, height_limit
real   ,dimension(1:Max_no)::dbh_heartwood, dbh_sapwood, crown_diameter, crown_area
real   ,dimension(1:Max_no)::la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available
real   ,dimension(1:Max_no)::bole_x, bole_y, crown_x, crown_y, radius_limit
real   ,dimension(1:Max_no)::stat_leaf, stat_leaf_crowntop, mort_regu1, mort_regu2
real,dimension(1:Max_no,10)::stat_leaf_crownbottom   !stat_leaf of foliage bottom layers
real,dimension(1:Max_no, 7)::stat_leaf_RunningRecord !1wk running record of stat_leaf

real,dimension(PFT_no,365)::npp_RunningRecord
real,dimension(PFT_no,365)::stat_water_RunningRecord

real,dimension( 12)::aet
real,dimension( 20)::gdd_20yr_RunningRecord
real,dimension( 20)::tmp_coldest_RunningRecord
real,dimension( 20)::tmp_hottest_RunningRecord
real,dimension( 20)::tmp_ave_C3growth_RunningRecord
real,dimension( 20)::tmp_ave_C4growth_RunningRecord
real,dimension( 30)::par_RunningRecord
real,dimension(365)::tmp_soil_RunningRecord
real,dimension(365)::tmp_air_RunningRecord
real,dimension(365)::prec_RunningRecord
real,dimension(365)::pool_w1_RunningRecord
real,dimension(365)::par_grass_RunningRecord

real,dimension(2)::tmp_recorder_C3growth, tmp_recorder_C4growth
real,dimension(2)::gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_grass, lai_opt_grass
real,dimension(1:2,1:20)::lai_opt_grass_RunningRecord 

!Define augments
integer           Fn
integer           p, n ,i, j, k
character(len=3)  fname, string

!************************   Set I/O number   *************************
Fn = File_no(22)

!************************  Write data  *************************
write (Fn,*) phenology(:)
write (Fn,*) pft_exist(:)
write (Fn,*) tree_exist(:)

write (Fn,'(3(i5,1x),i7,1x,2(i5,1x) )'  ) type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd
write (Fn,'(4(f10.2,1x))') pool_w1, pool_w2, pool_w3, pool_snow
write (Fn,'(3(f17.1,1x))') pool_litter, pool_som_int, pool_som_slow
write (Fn,'(f8.5,1x,f9.1,1x,f9.1)') canopy_cond, gdd0, gdd5
write (Fn,'(4(f10.5,1x))') establish_regu, mort_regu3, moist_factor, fire_factor
write (Fn,'(2(f15.1,1x))') c_emission, c_uptake
write (Fn,'(2(f10.5,1x))') pheno_gdd, tmp_soil_month_ave
write (Fn,'(3(f10.5,1x))') tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave
write (Fn,'(5(f10.5,1x))') hydRecord_preci,hydRecord_runoff,hydRecord_ic,hydRecord_ev,hydRecord_tr
write (Fn,'(4(f10.5,1x))') lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass

i = int(  PFT_no / 100      ) ; string(1:1)=char(i+48)
j = int( (PFT_no-i*100)/10  ) ; string(2:2)=char(j+48)
k = int(  PFT_no-i*100-j*10 ) ; string(3:3)=char(k+48)

write (Fn,'('//string//'(i6,1x))'    ) pheno_clo(:)
write (Fn,'('//string//'(i6,1x))'    ) pheno_cls(:)
write (Fn,'('//string//'(f6.4,1x))'  ) stat_water(:)
write (Fn,'('//string//'(f10.1,1x))' ) npp_10d_ave(:)

Do i=1, 365
   write (Fn,'('//string//'(f10.1,1x))') npp_RunningRecord(:,i)
End do

Do i=1, 365
   write (Fn,'('//string//'(f5.3,1x))') stat_water_RunningRecord(:,i)
End do

write (Fn,'( 12(f7.2,1x))') aet(:)
write (Fn,'( 20(f7.2,1x))') gdd_20yr_RunningRecord(:)

write (Fn,'( 20(f6.2,1x))') tmp_coldest_RunningRecord(:)
write (Fn,'( 20(f6.2,1x))') tmp_hottest_RunningRecord(:)
write (Fn,'( 20(f6.2,1x))') tmp_ave_C3growth_RunningRecord(:)
write (Fn,'( 20(f6.2,1x))') tmp_ave_C4growth_RunningRecord(:)
write (Fn,'(365(f6.2,1x))') tmp_soil_RunningRecord(:)
write (Fn,'(365(f6.2,1x))') tmp_air_RunningRecord(:)

write (Fn,'(365(f6.2,1x))') prec_RunningRecord(:)
write (Fn,'(365(f6.1,1x))') pool_w1_RunningRecord(:)
write (Fn,'( 30(f6.1,1x))') par_RunningRecord(:)
write (Fn,'(365(f6.1,1x))') par_grass_RunningRecord(:)

write (Fn,'( 4(f10.2,1x))') tmp_recorder_C3growth(:), tmp_recorder_C4growth(:)
write (Fn,'(12(f12.3,1x))') gmass_leaf(:), gmass_root(:), gmass_available(:), gmass_stock(:), lai_grass(:), lai_opt_grass(:)
write (Fn,'(40(f5.2,1x))') lai_opt_grass_RunningRecord(1,:), lai_opt_grass_RunningRecord(2,:)

Do n=1, Max_no
if (tree_exist(n)) then
   write (Fn,'(5(i5,1x) )'  ) pft(n), age(n), height(n), bole(n), height_limit(n)
   write (Fn,'(4(f10.5,1x))') dbh_heartwood(n), dbh_sapwood(n), crown_diameter(n), crown_area(n)
   write (Fn,'(6(f12.2,1x))') la(n), mass_leaf(n), mass_trunk(n), mass_root(n), mass_stock(n), mass_available(n)
   write (Fn,'(5(f10.5,1x))') bole_x(n), bole_y(n), crown_x(n), crown_y(n), radius_limit(n)
   write (Fn,'(4(f10.5,1x))') stat_leaf(n), stat_leaf_crowntop(n), mort_regu1(n), mort_regu2(n)
   write (Fn,'(10(f10.5,1x))') stat_leaf_crownbottom(n,:)
   write (Fn,'( 7(f10.5,1x))') stat_leaf_RunningRecord(n,:)
endif
End do

END SUBROUTINE spinup_out




!*************************************************************************************************
! output spinup files
!*************************************************************************************************
SUBROUTINE spinup_in &
   (type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd, &
   phenology, pft_exist, &
   tree_exist, &
   pool_w1, pool_w2, pool_w3, pool_snow, &
   pool_litter, pool_som_int, pool_som_slow, &
   canopy_cond, gdd0, gdd5, &
   establish_regu, mort_regu3, moist_factor, fire_factor, &
   c_emission, c_uptake, &
   pheno_gdd, tmp_soil_month_ave, &
   tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr, &
   lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
   pheno_clo, pheno_cls, &
   stat_water, npp_10d_ave, &
   pft, age, height, bole, height_limit, &
   dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, &
   la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, crown_x, crown_y, radius_limit, &
   stat_leaf, stat_leaf_crowntop, mort_regu1, mort_regu2, &
   stat_leaf_crownbottom, stat_leaf_RunningRecord, &
   npp_RunningRecord, stat_water_RunningRecord, &
   aet, gdd_20yr_RunningRecord, tmp_coldest_RunningRecord, tmp_hottest_RunningRecord, &
   tmp_ave_C3growth_RunningRecord, tmp_ave_C4growth_RunningRecord, &
   par_RunningRecord, tmp_soil_RunningRecord, tmp_air_RunningRecord, &
   prec_RunningRecord, pool_w1_RunningRecord, par_grass_RunningRecord, &
   tmp_recorder_C3growth, tmp_recorder_C4growth, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_grass, lai_opt_grass, &
   lai_opt_grass_RunningRecord)
   

USE data_structure
implicit none

!************************   DEFINE AUGMENTS   *************************
integer type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd

logical,dimension(PFT_no)::phenology, pft_exist
logical,dimension(1:Max_no)::tree_exist

real pool_w1, pool_w2, pool_w3, pool_snow
real pool_litter, pool_som_int, pool_som_slow
real canopy_cond, gdd0, gdd5
real establish_regu, mort_regu3, moist_factor, fire_factor
real c_emission, c_uptake
real pheno_gdd, tmp_soil_month_ave
real tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave
real hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr
real lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass

integer,dimension(PFT_no)::pheno_clo, pheno_cls
real   ,dimension(PFT_no)::stat_water, gpp, npp, gpp_month, npp_month
real   ,dimension(PFT_no)::gpp_annual, npp_annual, npp_10d_ave

integer,dimension(1:Max_no)::pft, age, height, bole, height_limit
real   ,dimension(1:Max_no)::dbh_heartwood, dbh_sapwood, crown_diameter, crown_area
real   ,dimension(1:Max_no)::la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available
real   ,dimension(1:Max_no)::bole_x, bole_y, crown_x, crown_y, radius_limit
real   ,dimension(1:Max_no)::stat_leaf, stat_leaf_crowntop, mort_regu1, mort_regu2
real,dimension(1:Max_no,10)::stat_leaf_crownbottom   !stat_leaf of foliage bottom layers
real,dimension(1:Max_no, 7)::stat_leaf_RunningRecord !1wk running record of stat_leaf

real,dimension(PFT_no,365)::npp_RunningRecord
real,dimension(PFT_no,365)::stat_water_RunningRecord

real,dimension( 12)::aet
real,dimension( 20)::gdd_20yr_RunningRecord
real,dimension( 20)::tmp_coldest_RunningRecord
real,dimension( 20)::tmp_hottest_RunningRecord
real,dimension( 20)::tmp_ave_C3growth_RunningRecord
real,dimension( 20)::tmp_ave_C4growth_RunningRecord
real,dimension( 30)::par_RunningRecord
real,dimension(365)::tmp_soil_RunningRecord
real,dimension(365)::tmp_air_RunningRecord
real,dimension(365)::prec_RunningRecord
real,dimension(365)::pool_w1_RunningRecord
real,dimension(365)::par_grass_RunningRecord

real,dimension(2)::tmp_recorder_C3growth, tmp_recorder_C4growth
real,dimension(2)::gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_grass, lai_opt_grass
real,dimension(1:2,1:20)::lai_opt_grass_RunningRecord 

!Define augments
integer           Fn
integer           p, n ,i, j, k
character(len=3)  fname, string

!************************   Set I/O number   *************************
Fn = File_no(21)

!************************  Read data  *************************
read (Fn,*) phenology (:)
read (Fn,*) pft_exist (:)
read (Fn,*) tree_exist(:)

read (Fn,*) type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd
read (Fn,*) pool_w1, pool_w2, pool_w3, pool_snow
read (Fn,*) pool_litter, pool_som_int, pool_som_slow
read (Fn,*) canopy_cond, gdd0, gdd5
read (Fn,*) establish_regu, mort_regu3, moist_factor, fire_factor
read (Fn,*) c_emission, c_uptake
read (Fn,*) pheno_gdd, tmp_soil_month_ave
read (Fn,*) tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave
read (Fn,*) hydRecord_preci,hydRecord_runoff,hydRecord_ic,hydRecord_ev,hydRecord_tr
read (Fn,*) lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass

i = int(  PFT_no / 100      ) ; string(1:1)=char(i+48)
j = int( (PFT_no-i*100)/10  ) ; string(2:2)=char(j+48)
k = int(  PFT_no-i*100-j*10 ) ; string(3:3)=char(k+48)

read (Fn,*) pheno_clo(:)
read (Fn,*) pheno_cls(:)
read (Fn,*) stat_water(:)
read (Fn,*) npp_10d_ave(:)

Do i=1, 365
   read (Fn,*) npp_RunningRecord(:,i)
End do

Do i=1, 365
   read (Fn,*) stat_water_RunningRecord(:,i)
End do

read (Fn,*) aet(:)
read (Fn,*) gdd_20yr_RunningRecord(:)

read (Fn,*) tmp_coldest_RunningRecord(:)
read (Fn,*) tmp_hottest_RunningRecord(:)
read (Fn,*) tmp_ave_C3growth_RunningRecord(:)
read (Fn,*) tmp_ave_C4growth_RunningRecord(:)
read (Fn,*) tmp_soil_RunningRecord(:)
read (Fn,*) tmp_air_RunningRecord(:)

read (Fn,*) prec_RunningRecord(:)
read (Fn,*) pool_w1_RunningRecord(:)
read (Fn,*) par_RunningRecord(:)
read (Fn,*) par_grass_RunningRecord(:)

read (Fn,*) tmp_recorder_C3growth(:), tmp_recorder_C4growth(:)
read (Fn,*) gmass_leaf(:), gmass_root(:), gmass_available(:), gmass_stock(:), lai_grass(:), lai_opt_grass(:)
read (Fn,*) lai_opt_grass_RunningRecord(1,:), lai_opt_grass_RunningRecord(2,:)

Do n=1, Max_no
if (tree_exist(n)) then
   read (Fn,*) pft(n), age(n), height(n), bole(n), height_limit(n)
   read (Fn,*) dbh_heartwood(n), dbh_sapwood(n), crown_diameter(n), crown_area(n)
   read (Fn,*) la(n), mass_leaf(n), mass_trunk(n), mass_root(n), mass_stock(n), mass_available(n)
   read (Fn,*) bole_x(n), bole_y(n), crown_x(n), crown_y(n), radius_limit(n)
   read (Fn,*) stat_leaf(n), stat_leaf_crowntop(n), mort_regu1(n), mort_regu2(n)
   read (Fn,*) stat_leaf_crownbottom(n,:)
   read (Fn,*) stat_leaf_RunningRecord(n,:)
endif
End do


END SUBROUTINE spinup_in


!*************************************************************************************************
! update size class distribution
!*************************************************************************************************
SUBROUTINE size_class_update &
           (size_dist, tree_exist, pft, dbh_heartwood, dbh_sapwood)

USE data_structure
implicit none

integer,dimension(21,PFT_no),intent(OUT)::size_dist     !size class distribution for each woody PFT
logical,dimension(1:Max_no) ,intent(IN) ::tree_exist    !tree exist flag
integer,dimension(1:Max_no) ,intent(IN) ::pft           !plant functional type
real   ,dimension(1:Max_no) ,intent(IN) ::dbh_heartwood !heartwood diameter at 1.3m (m)
real   ,dimension(1:Max_no) ,intent(IN) ::dbh_sapwood   !sapwood   diameter at 1.3m (m)

!For general usage
integer no, size
real    x

!Reset size distribution
size_dist(:,:) = 0.0

!Count size distribution
Do no=1, Max_no
   if ( .not. tree_exist(no) ) cycle
   
   x = dbh_heartwood(no) + dbh_sapwood(no)
   
                  size =  1 !     ~5 cm
   if (x >  0.05) size =  2 !   5~10 cm
   if (x >  0.10) size =  3 !  10~15 cm
   if (x >  0.15) size =  4 !  15~20 cm
   if (x >  0.20) size =  5 !  20~25 cm
   if (x >  0.25) size =  6 !  25~30 cm
   if (x >  0.30) size =  7 !  30~35 cm
   if (x >  0.35) size =  8 !  35~40 cm
   if (x >  0.40) size =  9 !  40~45 cm
   if (x >  0.45) size = 10 !  45~50 cm
   if (x >  0.50) size = 11 !  50~55 cm
   if (x >  0.55) size = 12 !  55~60 cm
   if (x >  0.60) size = 13 !  60~65 cm
   if (x >  0.65) size = 14 !  65~70 cm
   if (x >  0.70) size = 15 !  70~75 cm
   if (x >  0.75) size = 16 !  75~80 cm
   if (x >  0.80) size = 17 !  80~85 cm
   if (x >  0.85) size = 18 !  85~90 cm
   if (x >  0.90) size = 19 !  90~95 cm
   if (x >  0.95) size = 20 ! 95~100 cm
   if (x > 0.100) size = 21 ! 100~   cm
   
   size_dist(size,pft(no)) = size_dist(size,pft(no)) + 1
   
End Do

END SUBROUTINE size_class_update
